require 'kraftwerk/router'
require 'kraftwerk/app/configuration'
require 'kraftwerk/app/components'
require 'kraftwerk/app/dependencies'
require 'hanami/middleware/body_parser'
require 'kraftwerk/middleware/reloader'
require 'hanami/utils/class_attribute'
require 'dry/container'
require 'dry/auto_inject'

module Kraftwerk
  class App
    RoutingAlreadyDefined = Class.new(StandardError)
    DependenciesAlreadyDefined = Class.new(StandardError)
    AlreadyConfigured = Class.new(StandardError)

    include Hanami::Utils::ClassAttribute
    class_attribute :_dependencies, :_routing, :_zeitwerk_loader

    # configuration
    class_attribute :autoload_paths, :reloading_enabled, :configuration

    def self.inherited(base)
      create_configuration!(base)
    end

    # Initializes all the machinery
    # Should only be called once
    def self.initialize!
      self::Configuration.freeze
      create_components!

      require 'kraftwerk/logger/dev_logger'
      logger = Kraftwerk::Logger::DevLogger.new($stdout)

      self::Components['telemetry'].attach(
        'kraftwerk-request-start-log',
        [:kraftwerk, :request, :start],
        &logger.method(:handle_request_start)
      )
      self::Components['telemetry'].attach(
        'kraftwerk-request-finish-log',
        [:kraftwerk, :request, :finish],
        &logger.method(:handle_request_finish)
      )
    end

    class << self
      def routing(&block)
        raise RoutingAlreadyDefined unless self::Configuration.routing.nil?
        self::Configuration.routing Kraftwerk::Router.new(&Proc.new(&block))
      end

      def dependencies(&block)
        return self.create_dependencies!(&Proc.new(&block)) unless defined?(self::Dependencies)
        
        raise DependenciesAlreadyDefined if block_given?
        self::Dependencies
      end

      def rack_app
        self::Components[:rack_app]
      end
    end
  end
end
