require 'dry/container'
require 'kraftwerk/telemetry'
require 'kraftwerk/middleware/request_telemetry'

module Kraftwerk
  class App
    # Creates components
    # Configuration must be present and final at this point
    def self.create_components!
      kraftwerk_app = self
      config = self::Configuration
      klass = Class.new do
        extend Dry::Container::Mixin

        # Rack application for use in config.ru
        register(:rack_app, memoize: true) do
          router = resolve(:router)
          reloader = resolve(:reloader)

          Rack::Builder.new do
            use Kraftwerk::Middleware::Reloader.new(reloader)
            use Hanami::Middleware::BodyParser, :json
            use Kraftwerk::Middleware::RequestTelemetry, kraftwerk_app
            run router
          end
        end

        register(:router, memoize: true) do
          kraftwerk_app::Configuration.routing.routes
        end

        register(:reloader, memoize: true) do
          if config.reloading_enabled
            require 'zeitwerk'
            loader = Zeitwerk::Loader.new
            config.autoload_paths.each { |path| loader.push_dir(path) }
            loader.enable_reloading
            loader.setup
            loader
          else
            # fake reloader
            Class.new do
              def reload; end
            end
          end
        end

        register(:telemetry, memoize: true) do
          Kraftwerk::Telemetry.new
        end
      end

      self.const_set('Components', klass)
    end
  end
end
