module Kraftwerk
  module Endpoint
    module ErrorHandling
      def call(params)
        begin
          super
        rescue => exception
          if self.class.exception_handlers.key?(exception.class)
            handler = self.class.exception_handlers[exception.class]
            message = handler[:message].respond_to?(:call) ? handler[:message].call(exception) : handler[:message]
            unless message.is_a?(Kraftwerk::Response)
              message = Kraftwerk::Response.new(code: handler[:code], body: { error: message })
            end
            message
            # todo call app expection handler somehow
          else
            raise exception
          end
        end
      end

      def self.prepended(base)
        base.class_eval do
          class_attribute :exception_handlers
          self.exception_handlers = {}
          extend ClassMethods
        end
      end

      module ClassMethods
        def handle_exception(exception_class, code: 422, message: nil)
          body = message || Proc.new
          self.exception_handlers[exception_class] = { code: code, message: body }
        end
      end
    end
  end
end
