require 'dry/validation'
require 'kraftwerk/response'

module Kraftwerk
  module Endpoint
    module Validatable
      # Unlike in Hanami, where we usually want to validate params manually and, if they are invalid,
      # take a proper action, in Kraftwerk if validation is not passed, the request should not happen at all.
      # Instead, error messages are returned.
      def call(params)
        validation_class = self.class.validation_class
        return super if validation_class.nil?

        result = validation_class.new.call(params.to_h)
        if result.success?
          # pass only whitelisted parameters down
          super(result.to_h)
        else
          Response.new(code: 400, body: result.errors.to_h)
        end
      end

      def self.prepended(base)
        base.class_eval do
          class_attribute :validation_class
          extend ClassMethods
        end
      end

      module ClassMethods
        def contract(&block)
          klass = Class.new(Dry::Validation::Contract) do
            instance_exec(&Proc.new(&block))
          end
          self.validation_class = klass
        end

        def validate_with(klass)
          # TODO: add check if klass is proper validation class
          self.validation_class = klass
        end
      end
    end
  end
end
