# frozen_string_literal: true

require 'hanami/logger'

module Kraftwerk
  module Logger
    class DevLogger < ::Logger
      def initialize(*args)
        super
        self.formatter = proc do |severity, datetime, progname, message|
          message + "\n"
        end
      end

      def handle_request_start(data, meta)
        msg = String.new
        msg << "[#{meta[:time]}]\n"
        msg << color("[#{data[:method]}] ", :magenta)
        msg << color(data[:path], :green)
        msg << " "
        msg << (data[:params]&.empty? ? '' : JSON.dump(data[:params]))
        info(msg)
      end

      def handle_request_finish(data, meta)
        msg = String.new
        clr = data[:code] < 400 ? :green : :red

        msg << color("[#{data[:code]}] ", clr)
        msg << "[#{data[:duration].round(3)} ms] "
        if !data[:body].nil? && data[:body].length > 0
          msg << json_response(data[:body])
        else
          msg << "[empty response]"
        end

        info(msg + "\n")
      end

      private

      def color(text, color)
        Hanami::Utils::ShellColor.call(text, color: color)
      end

      def json_response(response)
        JSON.dump(JSON.parse(response))
      rescue
        "[non-JSON response: #{response}]"
      end
    end
  end
end
