# frozen_string_literal: true

module Kraftwerk
  # Based on
  # * BEAM Telemetry module
  # * Work of Tony Pitale: https://github.com/tpitale/telemetry-ruby
  class Telemetry
    HandlerIdAlreadyUsed = Class.new(RuntimeError)

    def initialize
      @handlers = Concurrent::Map.new
      @used_ids = Concurrent::Map.new
      @semaphore = Concurrent::Semaphore.new(1)
    end

    def attach(id, event_key, &handler)
      raise HandlerIdAlreadyUsed.new(id) if @used_ids.key?(id)

      @handlers[event_key] ||= Concurrent::Array.new
      @handlers[event_key] << { id: id, handler: handler }
      sync_used_ids_cache
    end

    def detach(id)
      event_key = @used_ids[id]
      @handlers[event_key].delete_if { |handler| handler[:id] == id }
      sync_used_ids_cache
    end

    def execute(event_key, values = {}, meta = {})
      handlers = @handlers[event_key]
      return if handlers.nil?

      handlers.each do |handler|
        handler[:handler].call(values, {_event_key: event_key}.merge(meta))
      end
    end

    private

    def sync_used_ids_cache
      ids = Concurrent::Map.new
      @semaphore.acquire
      @handlers.each do |key, handlers|
        handlers.each do |handler|
          id = handler[:id]
          ids[id] = key
        end
      end

      @used_ids = ids
    ensure
      @semaphore.release
    end
  end
end
